#include <algorithm>
#include <iostream>
#include <string>
#include <string_view>
#include <vector>

void show_size(const auto& str) {
	std::cout << "size: " << std::size(str) << '\n';
}

void range_for(const auto& str) {
	for (char c : str) {
		std::cout << "'" << c << "' ";
	}
	std::cout << '\n';
}

void copy_to_vec(const auto& str) {
	namespace ranges = std::ranges;

	std::vector<int> v;
	ranges::copy(str, std::back_inserter(v));

	for (int x : v) {
		std::cout << x << " ";
	}
	std::cout << '\n';
}

void show_search(const auto& haystack, const auto& needle) {
	namespace ranges = std::ranges;

	auto found = ranges::search(haystack, needle);

	if (found.empty()) {
		std::cout << "Not found.\n";
		return;
	}

	std::cout << "Found: \""
	          << std::string_view{ found.begin(), found.end() }
	          << "\"\n";
	// note: in C++23 we shall be able to write std::string_view{ found }
}

int main() {
	using namespace std::literals;

	show_size("lit");
	range_for("lit");
	copy_to_vec("lit");

	std::cout << "---\n";

	show_size("lit"sv);
	range_for("lit"sv);
	copy_to_vec("lit"sv);

	std::string text = "I must not fear. Fear is the mind-killer.";

	std::cout << "---\n";

	show_search(text, "fear"sv);
	show_search(text, "fear");
}
