#include <cassert>
#include <charconv>
#include <iostream>
#include <numbers>
#include <string>
#include <string_view>
#include <tuple>

void show(std::string_view s, std::to_chars_result r) {
	auto [ptr, ec] = r;
	if (ec != std::errc{}) {
		std::cout << std::make_error_code(ec).message() << '\n';
		return;
	}
	std::cout << "whole string: " << s
	          << "\n   until ptr: " << std::string_view{ s.data(), ptr }
	          << '\n';
}

void parse_all_ints(std::string_view s) {
	int x;
	auto ptr = s.data();
	auto end = s.data() + s.size();
	while (ptr != end) {
		auto fr = std::from_chars(ptr, end, x);
		if (fr.ec == std::errc{}) {
			std::cout << "got int: " << x
			          << "; rest of string: " << fr.ptr << '\n';
			ptr = fr.ptr;
		} else {
			std::cout << "no int found, skip char '"
			          << *ptr << "'\n";
			++ptr;
		}
	}
}


int main() {
	std::string s = "....................";

	auto r = std::to_chars(s.data(), s.data() + s.size(), 1337);
	show(s, r);

	r = std::to_chars(s.data(), s.data() + s.size(), std::numbers::pi);
	show(s, r);

	double my_pi;
        auto fr = std::from_chars(s.data(), r.ptr, my_pi);
	assert(fr.ec == std::errc{});
	assert(fr.ptr == r.ptr);
	assert(my_pi == std::numbers::pi);

	r = std::to_chars(s.data(), s.data() + s.size(), std::numbers::pi,
	                  std::chars_format::scientific, 4);
	show(s, r);

	parse_all_ints(s);
}
